﻿using System;
using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace KeepAppointmentsDistances
{
	public class CustomDragDropBehavior : ScheduleViewDragDropBehavior
	{
		public override bool CanDrop(DragDropState state)
		{
			var groupIteration = state.ServiceProvider.GetService<GroupIterationService>();

			var groups = groupIteration.GetGroups().ToArray();

			var sourceGroupIndex = GetGroupIndex(state.Appointment, groups);

			var draggedAppointmentIndex = IndexOf(state.DraggedAppointments, a => a == state.Appointment);
			var destinationSlot = state.DestinationSlots.ElementAtOrDefault(draggedAppointmentIndex);

			var destinationGroupIndex = GetGroupIndex(destinationSlot, groups);

			var offset = destinationGroupIndex - sourceGroupIndex;

			if (offset != 0)
			{
				foreach (var tuple in state.DraggedAppointments.Zip(state.DestinationSlots, (a, s) => new { Appointment = a, Slot = s }))
				{
					var sourceIndex = GetGroupIndex(tuple.Appointment, groups);
					tuple.Slot.Resources.Clear();
					tuple.Slot.Resources.Add(groups[sourceIndex + offset].Resources.FirstOrDefault());
				}
			}

			return base.CanDrop(state);
		}

		private static IResource GetResource(IOccurrence occurrence)
		{
			var occ = occurrence as Occurrence;
			var appointment = (occurrence as IAppointment) ?? (occ != null ? occ.Appointment : null);
			if (appointment != null)
			{
				return appointment.Resources.OfType<IResource>().FirstOrDefault();
			}
			return null;
		}

		private static int GetGroupIndex(IResource resource, IEnumerable<IGroupIdentifier> groups)
		{
			return IndexOf(groups, g => g.Resources.Contains(resource));
		}

		private static int GetGroupIndex(IOccurrence occurrence, IEnumerable<IGroupIdentifier> groups)
		{
			var resource = GetResource(occurrence);
			return GetGroupIndex(resource, groups);
		}

		private static int GetGroupIndex(Slot slot, IGroupIdentifier[] groups)
		{
			var resource = slot.Resources.OfType<IResource>().FirstOrDefault();
			return GetGroupIndex(resource, groups);
		}

		private static int IndexOf<T>(IEnumerable<T> items, Func<T, bool> predicate)
		{
			var i = 0;
			foreach (var item in items)
			{
				if (predicate(item))
				{
					return i;
				}
				i++;
			}

			return -1;
		}
	}
}